﻿Imports jp.co.systembase.report
Imports jp.co.systembase.report.component
Imports jp.co.systembase.report.renderer
Imports jp.co.systembase.report.renderer.gdi
Imports jp.co.systembase.report.renderer.gdi.elementrenderer

Public Class GdiImageRenderer
    Implements IElementRenderer

    Public Sub Render(
          ByVal env As RenderingEnv,
          ByVal reportDesign As ReportDesign,
          ByVal region As Region,
          ByVal design As ElementDesign,
          ByVal data As Object) Implements IElementRenderer.Render

        Dim _region As Region = region.ToPointScale(reportDesign)
        Dim g As System.Drawing.Graphics = env.Graphics

        If Not design.IsNull("fill_color") Then
            Dim color As System.Drawing.Color = RenderUtil.GetColor(design.Get("fill_color"))
            Using br As New System.Drawing.SolidBrush(color)
                g.FillRectangle(br, _region.Left, _region.Top, _region.GetWidth, _region.GetHeight)
            End Using
        End If

        Dim img As System.Drawing.Image = Nothing
        If Not env.InDesigner Then
            If Not design.IsNull("key") And data IsNot Nothing Then
                Dim key As String = design.Get("key")
                If env.ImageLoaderMap.ContainsKey(key) Then
                    img = env.ImageLoaderMap(key).GetImage(data)
                End If
            End If
        End If
        If img Is Nothing Then
            img = reportDesign.GetImage(design.Base, "image")
        End If
        If img Is Nothing Then
            Exit Sub
        End If

        Dim w As Single = img.Width
        Dim h As Single = img.Height
        Dim r As Single = 1.0F
        If w > _region.GetWidth Or h > _region.GetHeight Then
            r = Math.Min(_region.GetWidth / w, _region.GetHeight / h)
        End If
        If design.Get("fit") AndAlso
              (w < _region.GetWidth And h < _region.GetHeight) Then
            r = Math.Min(_region.GetWidth / w, _region.GetHeight / h)
        End If
        w *= r
        h *= r
        Dim t As Single = _region.Top
        Dim l As Single = _region.Left
        If Not design.IsNull("valign") Then
            Select Case design.Get("valign")
                Case "center"
                    t = _region.Top + (_region.GetHeight - h) / 2
                Case "bottom"
                    t = _region.Bottom - h
            End Select
        End If
        If Not design.IsNull("halign") Then
            Select Case design.Get("halign")
                Case "center"
                    l = _region.Left + (_region.GetWidth - w) / 2
                Case "right"
                    l = _region.Right - w
            End Select
        End If

        Dim opacity As Single = 1
        If Not design.IsNull("opacity") Then
            opacity = design.Get("opacity")
        End If

        Dim cm As New System.Drawing.Imaging.ColorMatrix()
        cm.Matrix00 = 1
        cm.Matrix11 = 1
        cm.Matrix22 = 1
        cm.Matrix33 = opacity
        cm.Matrix44 = 1
        Dim ia As New System.Drawing.Imaging.ImageAttributes()
        ia.SetColorMatrix(cm)

        g.DrawImage(img,
                    New System.Drawing.Rectangle(l, t, w, h),
                    0, 0, img.Width, img.Height,
                    System.Drawing.GraphicsUnit.Pixel,
                    ia)
    End Sub

End Class
